@@ -228,7 +228,7 @@ class SignedFunction(_function_base.Function):
     matcher = self.ctx.matcher(node)
     try:
       matches = matcher.compute_matches(
-          args_to_match, match_all_views, alias_map)
+          args_to_match, match_all_views, alias_map=alias_map)
     except matcher.MatchError as e:
       raise function.WrongArgTypes(self.signature, args, self.ctx, e.bad_type)
     return [m.subst for m in matches]
@@ -812,6 +812,10 @@ class InterpreterFunction(SignedFunction):
         }
         combinations = [combination]
         ret = self.ctx.new_unsolvable(node_after_call)
+      else:
+        if any(retval == self.ctx.convert.unsolvable
+               for retval in ret.Data(node_after_call)):
+          ret = self.ctx.new_unsolvable(node_after_call)
       for combination in combinations:
         for return_value in ret.bindings:
           values = list(combination.values()) + [return_value]
