@@ -3,7 +3,7 @@ import collections
 import contextlib
 import dataclasses
 import logging
-from typing import Any, Dict, Generic, List, Optional, Tuple, TypeVar
+from typing import Any, Dict, Iterable, List, Optional, Tuple
 
 from pytype import datatypes
 from pytype import special_builtins
@@ -28,8 +28,11 @@ _COMPATIBLE_BUILTINS = [
 ]
 
 _SubstType = datatypes.AliasingDict[str, cfg.Variable]
-_ViewType = Dict[cfg.Variable, cfg.Binding]
-_T = TypeVar("_T")
+_ViewType = datatypes.AccessTrackingDict[cfg.Variable, cfg.Binding]
+
+# For _UniqueMatches
+_ViewKeyType = Tuple[Tuple[int, Any], ...]
+_SubstKeyType = Dict[cfg.Variable, Any]
 
 
 def _is_callback_protocol(typ):
@@ -95,12 +98,12 @@ class GoodMatch:
 
   @classmethod
   def default(cls):
-    return cls({}, datatypes.HashableDict())
+    return cls(datatypes.AccessTrackingDict(), datatypes.HashableDict())
 
   @classmethod
   def merge(cls, old_match, new_match, combined_subst):
-    return cls({**old_match.view, **new_match.view},
-               datatypes.HashableDict(combined_subst))
+    view = datatypes.AccessTrackingDict.merge(old_match.view, new_match.view)
+    return cls(view, datatypes.HashableDict(combined_subst))
 
 
 @dataclasses.dataclass(eq=True, frozen=True)
@@ -129,38 +132,79 @@ class MatchResult:
   bad_matches: List[BadMatch]
 
 
-class _UniqueSubsts(Generic[_T]):
-  """A collection of substs that discards duplicates.
+class _UniqueMatches:
+  """A collection of matches that discards duplicates."""
 
-  Each subst has a bit of data associated to it, and calling unique() returns
-  unique substs and associated data.
-  """
+  def __init__(self, node, keep_all_views):
+    self._node = node
+    self._keep_all_views = keep_all_views
+    self._data: Dict[
+        _ViewKeyType, List[Tuple[_SubstKeyType, _ViewType, _SubstType]]
+    ] = collections.defaultdict(list)
 
-  def __init__(self):
-    self._data: List[Tuple[Dict[str, Any], _SubstType, _T]] = []
-
-  def insert(self, subst, data: _T):
+  def insert(self, view, subst):
     """Insert a subst with associated data."""
+    if self._keep_all_views:
+      view_key = tuple(sorted((k.id, v.data.get_type_key())
+                              for k, v in view.accessed_subset.items()))
+    else:
+      view_key = ()
     subst_key = {k: {v.get_type_key() for v in var.data}
                  for k, var in subst.items()}
-    data_item = (subst_key, subst, data)
-    for i, (prev_subst_key, *_) in enumerate(self._data):
+    data_item = (subst_key, view, subst)
+    for i, prev_data_item in enumerate(self._data[view_key]):
+      prev_subst_key, prev_view, prev_subst = prev_data_item
       if all(k in prev_subst_key and subst_key[k] <= prev_subst_key[k]
              for k in subst_key):
         # A previous substitution is a superset of this one, so we do not need
-        # to keep this one.
+        # to keep this one. We do copy over the view and origins.
+        prev_view.update(view)
+        for k, v in subst.items():
+          prev_subst[k].PasteVariable(v)
         break
       if all(k in subst_key and prev_subst_key[k] <= subst_key[k]
              for k in prev_subst_key):
         # This substitution is a superset of a previous one, so we replace the
-        # previous subst with this one.
-        self._data[i] = data_item
+        # previous subst with this one. We do copy over the view and origins.
+        self._data[view_key][i] = data_item
+        view.update(prev_view)
+        for k, v in prev_subst.items():
+          subst[k].PasteVariable(v)
         break
     else:
-      self._data.append(data_item)
+      self._data[view_key].append(data_item)
 
-  def unique(self) -> List[Tuple[_SubstType, _T]]:
-    return [d[1:] for d in self._data]
+  def unique(self) -> Iterable[Tuple[_ViewType, _SubstType]]:
+    for values in self._data.values():
+      for _, view, subst in values:
+        yield (view, subst)
+
+
+class _TypeParams:
+  """Collection of TypeParameter objects encountered during matching."""
+
+  def __init__(self):
+    self.seen = set()
+    self._mutually_exclusive = collections.defaultdict(set)
+
+  def add_mutually_exclusive_groups(self, groups):
+    """Adds groups of mutually exclusive type parameters.
+
+    For example, [{"T1", "T2"}, {"T3", "T4"}] would mean that the following
+    pairs are mutually exclusive: (T1, T3), (T1, T4), (T2, T3), (T2, T4).
+
+    Args:
+      groups: The mutually exclusive groups.
+    """
+    all_params = set.union(*groups)
+    for group in groups:
+      mutually_exclusive = all_params - group
+      for name in group:
+        self._mutually_exclusive[name].update(mutually_exclusive)
+
+  def has_mutually_exclusive(self, name, subst):
+    """Whether 'subst' has a param that is mutually exclusive with 'name'."""
+    return bool(self._mutually_exclusive[name].intersection(subst))
 
 
 class AbstractMatcher(utils.ContextWeakrefMixin):
@@ -183,7 +227,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
     self._recursive_annots_cache: Dict[
         Tuple[abstract.BaseValue, abstract.BaseValue], bool] = {}
     self._error_subst = None
-    self._seen_type_params = set()
+    self._type_params = _TypeParams()
     self._reset_errors()
 
   def _reset_errors(self):
@@ -269,8 +313,12 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
           subst[name] = value
     return datatypes.HashableDict(subst), None
 
+  # TODO(b/63407497): We were previously enforcing --strict_parameter_checks
+  # in compute_one_match, which didn't play nicely with overloads. Instead,
+  # enforcement should be pushed to callers of compute_matches.
   def compute_matches(
       self, args: List[function.Arg], match_all_views: bool,
+      keep_all_views: bool = False,
       alias_map: Optional[datatypes.UnionFind] = None) -> List[GoodMatch]:
     """Compute information about type parameters using one-way unification.
 
@@ -282,6 +330,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       match_all_views: If True, every possible match must succeed for the
         overall match to be considered a success. Otherwise, the overall match
         succeeds as long as at least one possible match succeeds.
+      keep_all_views: If True, avoid optimizations that discard views.
       alias_map: Optionally, a datatypes.UnionFind, which stores all the type
         renaming information, mapping of type parameter name to its
         representative.
@@ -294,26 +343,23 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
     has_self = args and args[0].name == "self"
     for arg in args:
       match_result = self.compute_one_match(
-          arg.value, arg.typ, arg.name, match_all_views, alias_map)
+          arg.value, arg.typ, arg.name, match_all_views,
+          keep_all_views, alias_map)
       if not match_result.success:
         if matches:
-          if self._error_subst:
-            self._error_subst = self._merge_substs(
-                matches[0].subst, [self._error_subst])
-          else:
-            self._error_subst = matches[0].subst
+          self._error_subst = matches[0].subst
           bad_param = self._get_bad_type(arg.name, arg.typ)
         else:
           bad_param = match_result.bad_matches[0].expected
         raise self.MatchError(bad_param)
-      if any(m.subst for m in match_result.good_matches):
+      if keep_all_views or any(m.subst for m in match_result.good_matches):
         matches = self._merge_matches(
             arg.name, arg.typ, matches, match_result.good_matches, has_self)
     return matches if matches else [GoodMatch.default()]
 
   def compute_one_match(
-      self, var, other_type, name=None, match_all_views=True, alias_map=None
-  ) -> MatchResult:
+      self, var, other_type, name=None, match_all_views=True,
+      keep_all_views=False, alias_map=None) -> MatchResult:
     """Match a Variable against a type.
 
     Args:
@@ -323,6 +369,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       match_all_views: If True, every possible match must succeed for the
         overall match to be considered a success. Otherwise, the overall match
         succeeds as long as at least one possible match succeeds.
+      keep_all_views: If True, avoid optimizations that discard views.
       alias_map: Optionally, a datatypes.UnionFind, which stores all the type
         renaming information, mapping of type parameter name to its
         representative.
@@ -330,7 +377,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       The match result.
     """
     bad_matches = []
-    good_matches = _UniqueSubsts[_ViewType]()
+    good_matches = _UniqueMatches(self._node, keep_all_views)
     views = abstract_utils.get_views([var], self._node)
     skip_future = None
     while True:
@@ -341,7 +388,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       subst = datatypes.AliasingDict(aliases=alias_map)
       subst = self.match_var_against_type(var, other_type, subst, view)
       if subst is None:
-        if self._node.HasCombination(list(view.values())):
+        if self._node.CanHaveCombination(list(view.values())):
           bad_matches.append(BadMatch(
               view=view,
               expected=self._get_bad_type(name, other_type),
@@ -351,15 +398,19 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         skip_future = False
       else:
         skip_future = True
-        good_matches.insert(subst, view)
+        good_matches.insert(view, subst)
     good_matches = [GoodMatch(view, datatypes.HashableDict(subst))
-                    for subst, view in good_matches.unique()]
-    if not bad_matches:
+                    for view, subst in good_matches.unique()]
+    if (good_matches and not match_all_views) or not bad_matches:
       success = True
-    elif match_all_views or self.ctx.options.strict_parameter_checks:
-      success = False
+    elif good_matches:
+      # Use HasCombination, which is much more expensive than
+      # CanHaveCombination, to re-filter bad matches.
+      bad_matches = [m for m in bad_matches
+                     if self._node.HasCombination(list(m.view.values()))]
+      success = not bad_matches
     else:
-      success = bool(good_matches)
+      success = False
     return MatchResult(
         success=success, good_matches=good_matches, bad_matches=bad_matches)
 
@@ -417,7 +468,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         right_side_options = [other_type]
       for right in right_side_options:
         if isinstance(right, abstract.TypeParameter):
-          self._seen_type_params.add(right)
+          self._type_params.seen.add(right)
           # If we have a union like "K or V" and we match both against
           # nothing, that will fill in both K and V.
           if right.full_name not in subst:
@@ -510,7 +561,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
                                    function.Signature)) or
         left.instance is abstract_utils.DUMMY_CONTAINER):
       if isinstance(other_type, abstract.TypeParameter):
-        self._seen_type_params.add(other_type)
+        self._type_params.seen.add(other_type)
         new_subst = self._match_type_param_against_type_param(
             left.param, other_type, subst, view)
         if new_subst is not None:
@@ -539,7 +590,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         # left to its upper bound.
         return self._instantiate_and_match(left.param, other_type, subst, view)
     elif isinstance(other_type, abstract.TypeParameter):
-      self._seen_type_params.add(other_type)
+      self._type_params.seen.add(other_type)
       for c in other_type.constraints:
         new_subst = self._match_value_against_type(value, c, subst, view)
         if new_subst is not None:
@@ -591,22 +642,30 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       # otherwise preserve the original order.
       options = sorted(enumerate(other_type.options),
                        key=lambda itm: (itm[1].formal, itm[0]))
+      new_substs = []
+      type_param_groups = []
       for _, t in options:
         new_subst = self._match_value_against_type(value, t, subst, view)
         if new_subst is None:
           continue
         matched = True
-        subst = new_subst
         if isinstance(value.data, abstract.AMBIGUOUS_OR_EMPTY) or t.formal:
+          new_substs.append(new_subst)
+          type_param_groups.append(set(new_subst).difference(subst))
           continue
         # Since options without type parameters do not modify subst, we can
         # break after the first match rather than finding all matches. We still
         # need to fill in subst with *something* so that
         # annotation_utils.sub_one_annotation can tell that all annotations have
         # been fully matched.
-        subst = self._subst_with_type_parameters_from(subst, other_type)
+        subst = self._subst_with_type_parameters_from(new_subst, other_type)
         break
-      return subst if matched else None
+      if not matched:
+        return None
+      if not new_substs:
+        return subst
+      self._type_params.add_mutually_exclusive_groups(type_param_groups)
+      return self._merge_substs(subst, new_substs)
     elif (isinstance(other_type, (abstract.Unknown, abstract.Unsolvable)) or
           isinstance(left, (abstract.Unknown, abstract.Unsolvable))):
       # We can match anything against unknown types, and unknown types against
@@ -665,8 +724,9 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
                 has_error = False
               elif old_value.cls.is_protocol:
                 with self._track_partially_matched_protocols():
+                  protocol_subst = datatypes.AliasingDict(subst)
                   has_error = self._match_against_protocol(
-                      new_value, old_value.cls, subst, {}) is None
+                      new_value, old_value.cls, protocol_subst, {}) is None
               if not has_error:
                 break
       else:
@@ -820,7 +880,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
     return self._merge_substs(subst, new_substs)
 
   def _mutate_type_parameters(self, params, value, subst):
-    self._seen_type_params.update(params)
+    self._type_params.seen.update(params)
     new_subst = {p.full_name: value.to_variable(self._node) for p in params}
     return self._merge_substs(subst, [new_subst])
 
@@ -862,7 +922,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
           not isinstance(right, abstract.TypeParameter) or
           right.constraints or right.bound or callable_param_count[right] != 1):
         return None
-      self._seen_type_params.add(right)
+      self._type_params.seen.add(right)
       subst = subst.copy()
       # We don't know what to fill in here, since we have a TypeVar matching a
       # TypeVar, but we have to add *something* to indicate the match succeeded.
@@ -923,15 +983,17 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
       self, name: str, formal: abstract.BaseValue,
       old_matches: Optional[List[GoodMatch]], new_matches: List[GoodMatch],
       has_self: bool) -> List[GoodMatch]:
-    if old_matches is None:
+    if not old_matches:
       return new_matches
+    if not new_matches:
+      return old_matches
     combined_matches = []
+    matched = False
+    bad_param = None
     for new_match in new_matches:
       cur_types = datatypes.AliasingDict(
-          {t.full_name: t for t in self._seen_type_params},
+          {t.full_name: t for t in self._type_params.seen},
           aliases=new_match.subst.aliases)
-      matched = False
-      bad_param = None
       for old_match in old_matches:
         combined_subst = self._match_subst_against_subst(
             old_match.subst, new_match.subst, cur_types, has_self)
@@ -943,8 +1005,8 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
         combined_matches.append(
             GoodMatch.merge(old_match, new_match, combined_subst))
         matched = True
-      if not matched:
-        raise self.MatchError(bad_param)
+    if not matched:
+      raise self.MatchError(bad_param)
     return combined_matches
 
   def _match_subst_against_subst(
@@ -970,11 +1032,16 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
           new_var, has_error = self._check_type_param_consistency(
               b2.data, b2, type_param_map[t],
               old_subst.copy(t=b1.AssignToNewVariable(self._node)))
-          if has_error:
+          # If new_subst contains a TypeVar that is mutually exclusive with t,
+          # then we can ignore this error because it is legal for t to not be
+          # present in new_subst.
+          ignore_error = self._type_params.has_mutually_exclusive(t, new_subst)
+          if has_error and not ignore_error:
             continue
+          keep_old = keep_old_values or ignore_error
           if t not in subst:
-            subst[t] = old_subst[t] if keep_old_values else new_var
-          elif not keep_old_values:
+            subst[t] = old_subst[t] if keep_old else new_var
+          elif not keep_old:
             subst[t].PasteVariable(new_var, self._node)
       if t not in subst:
         return None
@@ -1513,7 +1580,7 @@ class AbstractMatcher(utils.ContextWeakrefMixin):
   def _subst_with_type_parameters_from(self, subst, typ):
     subst = subst.copy()
     for param in self.ctx.annotation_utils.get_type_parameters(typ):
-      self._seen_type_params.add(param)
+      self._type_params.seen.add(param)
       if param.full_name not in subst:
         subst[param.full_name] = self.ctx.convert.empty.to_variable(self._node)
     return subst
