@@ -12,45 +12,24 @@ locally or within a larger repository.
 
 import collections
 import difflib
-import gzip
 import io
 import itertools
 import pickle
 import pickletools
 import re
-import sys
 
-from pytype import pytype_source_utils
 from pytype import utils
-from pytype.platform_utils import path_utils
 from pytype.pytd import printer
 from pytype.pytd import pytd
 from pytype.pytd import pytd_visitors
 from pytype.pytd.parse import parser_constants
 
 
-_PICKLE_PROTOCOL = pickle.HIGHEST_PROTOCOL
-_PICKLE_RECURSION_LIMIT_AST = 40000
-PICKLE_EXT = ".pickled"
-
 ANON_PARAM = re.compile(r"_[0-9]+")
 
 _TUPLE_NAMES = ("builtins.tuple", "typing.Tuple")
 
 
-class LoadPickleError(Exception):
-  """Errors when loading a pickled pytd file."""
-
-  def __init__(self, filename):
-    self.filename = filename
-    msg = f"Error loading pickle file: {filename}"
-    super().__init__(msg)
-
-
-def IsPickle(filename):
-  return path_utils.splitext(filename)[1].startswith(PICKLE_EXT)
-
-
 def UnpackUnion(t):
   """Return the type list for union type, or a list with the type itself."""
   if isinstance(t, pytd.UnionType):
@@ -355,68 +334,6 @@ class OrderedSet(dict):
     self[item] = None
 
 
-def GetPredefinedFile(stubs_subdir, module, extension=".pytd",
-                      as_package=False):
-  """Get the contents of a predefined PyTD, typically with a file name *.pytd.
-
-  Arguments:
-    stubs_subdir: the directory, typically "builtins" or "stdlib"
-    module: module name (e.g., "sys" or "__builtins__")
-    extension: either ".pytd" or ".py"
-    as_package: try the module as a directory with an __init__ file
-  Returns:
-    The contents of the file
-  Raises:
-    IOError: if file not found
-  """
-  parts = module.split(".")
-  if as_package:
-    parts.append("__init__")
-  mod_path = path_utils.join(*parts) + extension
-  path = path_utils.join("stubs", stubs_subdir, mod_path)
-  return path, pytype_source_utils.load_text_file(path)
-
-
-def _LoadPickle(f, filename):
-  """Load a pickle file, raising a custom exception on failure."""
-  try:
-    return pickle.load(f)
-  except Exception as e:  # pylint: disable=broad-except
-    raise LoadPickleError(filename) from e
-
-
-def LoadPickle(filename, compress=False, open_function=open):
-  with open_function(filename, "rb") as fi:
-    if compress:
-      with gzip.GzipFile(fileobj=fi) as zfi:
-        return _LoadPickle(zfi, filename)
-    else:
-      return _LoadPickle(fi, filename)
-
-
-def SavePickle(data, filename=None, compress=False, open_function=open):
-  """Pickle the data."""
-  recursion_limit = sys.getrecursionlimit()
-  sys.setrecursionlimit(_PICKLE_RECURSION_LIMIT_AST)
-  assert not compress or filename, "gzip only supported with a filename"
-  try:
-    if compress:
-      with open_function(filename, mode="wb") as fi:
-        # We blank the filename and set the mtime explicitly to produce
-        # deterministic gzip files.
-        with gzip.GzipFile(filename="", mode="wb",
-                           fileobj=fi, mtime=1.0) as zfi:
-          # TODO(b/173150871): Remove disable once typeshed bug is fixed.
-          pickle.dump(data, zfi, _PICKLE_PROTOCOL)  # pytype: disable=wrong-arg-types
-    elif filename is not None:
-      with open_function(filename, "wb") as fi:
-        pickle.dump(data, fi, _PICKLE_PROTOCOL)
-    else:
-      return pickle.dumps(data, _PICKLE_PROTOCOL)
-  finally:
-    sys.setrecursionlimit(recursion_limit)
-
-
 def ASTeq(ast1, ast2):
   return (ast1.constants == ast2.constants and
           ast1.type_params == ast2.type_params and
